import { Component, Input, OnInit } from '@angular/core';
import * as Prism from 'prismjs';
import 'prismjs/components/prism-scss';
import 'prismjs/components/prism-c';
import 'prismjs/components/prism-cpp';
import 'prismjs/components/prism-java';
import 'prismjs/components/prism-python';
import 'prismjs/components/prism-markdown';
import { LanguageDefinition } from 'prismjs';
import { IGitFiles } from 'app/shared/model/git-files.model';

@Component({
  selector: 'jhi-home-highlighting',
  templateUrl: './highlighting.component.html',
  styleUrls: ['./highlighting.scss'],
})
export class HighlightingComponent implements OnInit {
  @Input() gitFiles: IGitFiles | undefined;
  public language: LanguageDefinition | undefined;

  private static highlight(content: string, language: LanguageDefinition): string {
    const elements = content.split(/(<mark><strong>|<\/strong><\/mark>)/g);
    for (let i = 0; i < elements.length; ++i) {
      if (elements[i] === '<mark><strong>' || elements[i] === '</strong></mark>') {
        continue;
      }
      elements[i] = Prism.highlight(elements[i], language, language);
    }
    return elements.join('');
  }

  ngOnInit(): void {
    this.language = undefined;
    switch (this.gitFiles?.fileFormat) {
      case 'C':
        this.language = Prism.languages.c;
        break;
      case 'C++':
        this.language = Prism.languages.cpp;
        break;
      case 'Java':
        this.language = Prism.languages.java;
        break;
      case 'Python':
        this.language = Prism.languages.python;
        break;
      case 'Markdown':
        this.language = Prism.languages.markdown;
        break;
    }
  }

  public highlightCode(line: string): string {
    if (!this.gitFiles) {
      return '';
    }
    if (this.language) {
      return HighlightingComponent.highlight(line, this.language);
    }
    return line;
  }

  public lineByLine(): string[] {
    return this.gitFiles?.fragment.fragment?.split(/\n/) || [];
  }

  lineNumber(i: number): string {
    if (this.language) {
      return String(i + (this.gitFiles?.fragment?.firstLineNumber || 0));
    }
    return ' ';
  }

  url(i: number): void {
    if (!this.gitFiles) {
      return;
    }
    window.location.href = this.gitFiles.gitUrl + '#L' + this.lineNumber(i);
  }
}
